import re
import numpy as np
import pandas as pd
from pathlib import Path

def get_annot_col(data_desc):
    
    """
    function to identify annotation column based on column description
    Input: gpl.columns
    Output: List of relevant column to be identified as annotation col
    """
    gene_annot_col=[]
    for idx in data_desc.index:
        if ("accession" in data_desc.loc[data_desc.index==idx]["description"].values[0]) or ("Accession" in data_desc.loc[data_desc.index==idx]["description"].values[0]):
            gene_annot_col.append(idx)
        elif ("Gene Name" in data_desc.loc[data_desc.index==idx]["description"].values[0]) or ("gene" in data_desc.loc[data_desc.index==idx]["description"].values[0]):
            gene_annot_col.append(idx)
        elif ("entrez" in data_desc.loc[data_desc.index==idx]["description"].values[0]) or ("Entrez" in data_desc.loc[data_desc.index==idx]["description"].values[0]):
            gene_annot_col.append(idx)
    return gene_annot_col

def genbank_acc(value):
    
    """
    function: Pattern matching for genebank accession numbers
    Input: String (column value)
    Output: Accession num, if identified correctly
    """
    regex1=r"(^[A-Z]{1,2}[0-9]{5,7}$)"
    regex2=r"(^[A-Z]{2}\_[0-9]{5,6}$)"
    regex3=r"(^[A-Z]{2}\_[0-9]{5,6}\.\d?$)"
    match=""
    if re.findall(regex1,value):
        match=re.findall(regex1,value)
    elif re.findall(regex2,value):
        match=re.findall(regex2,value)
    elif re.findall(regex3,value):
        match=re.findall(regex3,value)
    return match
    
def check_annotation_by_col(col_list,data):
    """
    function: Identify annotation col and type based on pattern 
    Input: relevant col list and GPL table
    Output: Dict of annot_col and annot_type
    
    """
    check_annot_col=[]
    for each_col in data[col_list].columns:
        symbo_entrez=pd.read_csv(str(Path(__file__).parent) +"/annotation_files/symbol-to-entrez.tsv",sep='\t',header=None)
        values=data[each_col].unique()[:20]
        for each_val in values:
            if (type(each_val)==str) and "//" in each_val:
                each_val=each_val.split("//")[0]
            elif (type(each_val)==str) and "," in each_val:
                each_val=each_val.split(",")[0]
            if (type(each_val)==str) and each_val.startswith("ENS"):
                check_annot_col.append([each_col,"ensembl"])
            elif (type(each_val)==str) and genbank_acc(each_val):
                if each_col == "ACCESSION_STRING":
                    pass
                else:
                    check_annot_col.append([each_col,"genbank"])
            elif (type(each_val)==str) and each_val in symbo_entrez[2].tolist():
                check_annot_col.append([each_col,"symbol"])
            elif ((type(each_val)==np.float64) or (type(each_val)==np.int64) ) and each_val in symbo_entrez[1].tolist():
                if each_col == "Sequence Length":
                    pass
                elif ((-1) <= int(each_val) <=9):
                    pass
                else:
                    check_annot_col.append([each_col,"entrez"])
    return check_annot_col

def find_annotation(data_desc,data):
    gene_annot_col=get_annot_col(data_desc)
    if gene_annot_col:
        check_annot_col=check_annotation_by_col(gene_annot_col,data)
    else:  
        check_annot_col=check_annotation_by_col(list(data.columns),data)
        
    if check_annot_col:
        unique_col=[]
        for col in check_annot_col:
            if col not in unique_col:
                unique_col.append(col)
        annot=unique_col[0]
        if annot:
            return annot
        else:
            return ["False","False"]
                    
